/*	$Workfile: i2c.h $
						  
	$Author: Andrey Volkov $ 		
	$Revision: 1 $             
	$Date: 31.10.01 17:34 $
*/                           

#ifndef __I2C_H__
#define __I2C_H__

#include "tpu.h"

template<class _T> class I2C_MasterBase
{
  enum 
  { 
   idle,
   SCL_HIGHT_BIT0, SCL_HIGHT_BIT1,SCL_HIGHT_BIT2, SCL_HIGHT_BIT3, SCL_HIGHT_BIT4,
   SCL_HIGHT_BIT5, SCL_HIGHT_BIT6, SCL_HIGHT_BIT7, SCL_HIGHT_ASK,

   SCL_LOW_BIT0,SCL_LOW_BIT1, SCL_LOW_BIT2, SCL_LOW_BIT3, SCL_LOW_BIT4,
   SCL_LOW_BIT5, SCL_LOW_BIT6, SCL_LOW_BIT7, SCL_LOW_ASK, 
   
   SCL_DELAY
  } m_state;

 BYTE isReadMode:1;
 BYTE Ack:1;

 BYTE *m_pData;
 BYTE  m_len;
 BYTE  m_count;
 BYTE  m_curByte;
public:
 void Start()
	{
	   ((_T *)this)->dataLow();
	   ((_T *)this)->dataOut();
	   DelayUs(Us2DelayClock(_T::longDelay));
	   ((_T *)this)->clockLow();
	   ((_T *)this)->clockOut();
	}

 void Stop() 
	{ 
	   ((_T *)this)->clockIn();
	   DelayUs(Us2DelayClock(_T::longDelay));
  	   ((_T *)this)->dataIn(); 
	} 

protected:
 typedef I2C_MasterBase<_T> _Parent;

 inline void OnTimer(void) 
 {
	I2C_TIMER.TSR.Bit.TGFA = 0;

	if( m_count && isReadMode )
	   goto read_i2c;

		switch( m_state ) 
		{
			case SCL_HIGHT_ASK :  m_state = SCL_LOW_ASK;
			   Ack = ((_T *)this)->getDataPin();  // Get Slave acknowelege ACK=low, NACK=high
			   ((_T *)this)->clockLow();
			   ((_T *)this)->dataOut();
			   break;
			   
			case SCL_HIGHT_BIT0 : m_state = SCL_LOW_BIT0; goto scl_low;
			case SCL_HIGHT_BIT1 : m_state = SCL_LOW_BIT1; goto scl_low;
			case SCL_HIGHT_BIT2 : m_state = SCL_LOW_BIT2; goto scl_low;
			case SCL_HIGHT_BIT3 : m_state = SCL_LOW_BIT3; goto scl_low;
			case SCL_HIGHT_BIT4 : m_state = SCL_LOW_BIT4; goto scl_low;
			case SCL_HIGHT_BIT5 : m_state = SCL_LOW_BIT5; goto scl_low; 
			case SCL_HIGHT_BIT6 : m_state = SCL_LOW_BIT6;
			scl_low:
				   ((_T *)this)->clockLow();
			   break;
			 
			case SCL_HIGHT_BIT7 : 
				   m_state = SCL_LOW_BIT7;
				   ((_T *)this)->clockLow();
				   ((_T *)this)->dataIn();      // release data line for get ACK
			   break;
			   
			case SCL_LOW_ASK :
			   if( Ack || ++m_count == m_len ) 
			   {  // abort read if NACK (high)
			      m_state = idle;      
			      Stop();
			      ((_T *)this)->TimerStop();
			      ((_T *)this)->Complete();
			      return;
			   }
			   m_pData++;
			   m_curByte = *m_pData;
			   if( isReadMode ) 
			   {
			      ((_T *)this)->dataIn();      // prepare for read
			      goto read_bit0;
			   } 
			   m_state = SCL_DELAY;   
			   break;
			   
			case SCL_DELAY :  m_state = SCL_HIGHT_BIT0; goto scl_hi;
			   
			case SCL_LOW_BIT0 : m_state = SCL_HIGHT_BIT1; goto scl_hi;
			case SCL_LOW_BIT1 : m_state = SCL_HIGHT_BIT2; goto scl_hi;
			case SCL_LOW_BIT2 : m_state = SCL_HIGHT_BIT3; goto scl_hi;
			case SCL_LOW_BIT3 : m_state = SCL_HIGHT_BIT4; goto scl_hi;
			case SCL_LOW_BIT4 : m_state = SCL_HIGHT_BIT5; goto scl_hi;
			case SCL_LOW_BIT5 : m_state = SCL_HIGHT_BIT6; goto scl_hi;
			case SCL_LOW_BIT6 : m_state = SCL_HIGHT_BIT7;
			scl_hi:
			   if( m_curByte & 0x80 )    // MSB first
			      ((_T *)this)->dataHigh(); 
			   else     
			      ((_T *)this)->dataLow();
			   m_curByte <<= 1;
			   DelayUs(Us2DelayClock(_T::longDelay));
			   ((_T *)this)->clockHigh();
			   break;
			   
			case SCL_LOW_BIT7 :
			   m_state = SCL_HIGHT_ASK;
			   ((_T *)this)->clockHigh();
			   break;
			   
			case idle:   
			   break;
		   }
	return;

	read_i2c:      /*-------------------- READ i2c */
		switch( m_state ) 
		{
			case SCL_HIGHT_BIT0 : m_state = SCL_LOW_BIT0; goto sclr_low;
			case SCL_HIGHT_BIT1 : m_state = SCL_LOW_BIT1; goto sclr_low;
			case SCL_HIGHT_BIT2 : m_state = SCL_LOW_BIT2; goto sclr_low;
			case SCL_HIGHT_BIT3 : m_state = SCL_LOW_BIT3; goto sclr_low;
			case SCL_HIGHT_BIT4 : m_state = SCL_LOW_BIT4; goto sclr_low;
			case SCL_HIGHT_BIT5 : m_state = SCL_LOW_BIT5; goto sclr_low; 
			case SCL_HIGHT_BIT6 : m_state = SCL_LOW_BIT6; goto sclr_low;
			case SCL_HIGHT_BIT7 : m_state = SCL_LOW_BIT7;
			sclr_low:
			   m_curByte >>= 1;
			   if( ((_T *)this)->getDataPin() )
			     m_curByte |= 0x80;
			   else    
			     m_curByte &= ~0x80;
			   ((_T *)this)->clockLow();
			   break;
			 
			case SCL_HIGHT_ASK :  m_state = SCL_LOW_ASK;
			   ((_T *)this)->dataLow();
			   ((_T *)this)->clockLow();
			   ((_T *)this)->dataIn();         // Release data line for slave output
			   break;

			case SCL_LOW_ASK :
			   if(++m_count == m_len ) 
			   {
			      m_state = idle;      
			      Stop();
			      ((_T *)this)->TimerStop();
			      ((_T *)this)->Complete();
			      return;
			   }
			   m_pData++;
			   m_curByte = 0;
			   
			read_bit0:   

			case SCL_LOW_BIT0 : m_state = SCL_HIGHT_BIT1; goto sclr_hi;
			case SCL_LOW_BIT1 : m_state = SCL_HIGHT_BIT2; goto sclr_hi;
			case SCL_LOW_BIT2 : m_state = SCL_HIGHT_BIT3; goto sclr_hi;
			case SCL_LOW_BIT3 : m_state = SCL_HIGHT_BIT4; goto sclr_hi;
			case SCL_LOW_BIT4 : m_state = SCL_HIGHT_BIT5; goto sclr_hi;
			case SCL_LOW_BIT5 : m_state = SCL_HIGHT_BIT6; goto sclr_hi;
			case SCL_LOW_BIT6 : m_state = SCL_HIGHT_BIT7;
			sclr_hi:
			   ((_T *)this)->clockHigh();
			   break;
			   
			case SCL_LOW_BIT7 : m_state = SCL_HIGHT_ASK;
			    ((_T *)this)->dataOut();
			   if( m_count == m_len - 1 )
			      ((_T *)this)->dataHigh();      // Set NACK on last received byte
			   else   
			      ((_T *)this)->dataLow();     // Set ACK on non last received byte
			   ((_T *)this)->clockHigh();
			   break;
			   
			case idle:   
			   break;
		   }
 }

public:
 typedef void (*callback)(_Parent *pThis);

 bool isBusy() {return m_state != idle;}

 void dataIn()  	{};
 void dataOut() 	{};
 void dataHigh()	{};
 void dataLow() 	{};

 void clockIn()   	{};
 void clockOut()  	{};
 void clockHigh() 	{};
 void clockLow()  	{};

 bool getDataPin()  { return true;};


 void TimerStart() {}
 void TimerStop()  {}

 void Complete() {};

 BYTE write( void* buffer, BYTE len )
 {
 	if( m_state != idle || len ==0)
 	   return 0;      

 	Start();

 	isReadMode = false;
 	m_len = len;
 	m_pData = (BYTE *)buffer;
 	m_count = 0;
 	m_curByte = *m_pData << 1; // bit0=0 for write

 	if( m_curByte & 0x80 )    // MSB first
 	   ((_T *)this)->dataHigh(); 
 	else     
 	   ((_T *)this)->dataLow(); 
 	m_curByte <<= 1;

 	m_state = SCL_HIGHT_BIT0;
     DelayUs(Us2DelayClock(_T::shortDelay));
    clockHigh(); 

	((_T *)this)->TimerStart();

 	return len;      
 }

 /*---------------------------------------------------------------------------*/

 BYTE read(void* buffer, BYTE len )
 {
 	if( m_state != idle || len ==0)
 	   return 0;      

 	Start();

 	isReadMode = 1;

 	m_len = len;
 	m_pData = (BYTE *)buffer;
 	m_count = 0;
 	m_curByte = ( *m_pData << 1 ) | 1; // bit0=1 for read

 	if( m_curByte & 0x80 )    	// MSB first
 	   ((_T *)this)->dataHigh(); 
 	else     
 	   ((_T *)this)->dataLow(); 

 	m_curByte <<= 1;

 	m_state = SCL_HIGHT_BIT0;
    DelayUs(Us2DelayClock(_T::shortDelay));
    ((_T *)this)->clockLow();
 	((_T *)this)->TimerStart();

 	return len;      
 }
};


#endif
