#ifndef __LM87_H__
#define __LM87_H__

#include <i2c.h>

class CLm87: public I2C_MasterBase<CLm87> 
{
public:
//I2C delays
  enum {shortDelay=1, longDelay=5};

  enum {i2cAddr=0x2c};

//LM87 regs
  enum regs 
        {
        IntTempHiLimitWithLock=0x13, 
  	    ExtTempHiLimitWithLock,Test,ChannelMode,IntTempHiLimit,ExtTempHiLimit,Dac,
		MonitorRes,
		VendId=0x3e,RevId,
  	    CFG1,ISR1,ISR2,IMR1,IMR2,
  	    CiClr=0x46,VID0_3,VID4=0x49,
		CFG2,
		ISR1Mirror=0x4c,ISR2Mirror,
		SMBALERT=0x80
  	    };

  CLm87() 
  {
	I2C_TIMER.TCR.Bit.CCLR = 1; // TCNT clearing other chanel 4
	I2C_TIMER.TCR.Bit.CKEG = 0; // Count at rising edge
	I2C_TIMER.TCR.Bit.TPSC = 1; // Internal clock: counts on CLK/4

	I2C_TIMER.TMDR.Bit.BFB = 0; // TGRB operates normally
	I2C_TIMER.TMDR.Bit.BFA = 0; // TGRA operates normally
	I2C_TIMER.TMDR.Bit.MD  = 0;  // Normal operation

	I2C_TIMER.TIOR.Bit.IOB = 0;  //
	I2C_TIMER.TIOR.Bit.IOA = 0;  // 
	I2C_TIMER.TIOR.Bit.IOD = 0;  // 
	I2C_TIMER.TIOR.Bit.IOC = 0;  // 

	I2C_TIMER.TIER.Bit.TTGE  = 0;  // interrupt disabled
	I2C_TIMER.TIER.Bit.TCIEV = 0;  // 
	I2C_TIMER.TIER.Bit.TGIED = 0;  // 
	I2C_TIMER.TIER.Bit.TGIEC = 0;  // 
	I2C_TIMER.TIER.Bit.TGIEB = 0;  // 
	I2C_TIMER.TIER.Bit.TGIEA = 1;  // 

	I2C_TIMER.TGRA = TPU_TGR_SET( 4, 10000 ); // 10KHz timer
	I2C_TIMER.TGRB = 0;
	I2C_TIMER.TGRC = 0;
	I2C_TIMER.TGRD = 0;
	I2C_TIMER.TCNT = 0;

  };

  void dataIn()  	{I2C_DATA_IN();};
  void dataOut() 	{I2C_DATA_OUT();};
  void dataHigh()	{I2C_DATA_ON();};
  void dataLow() 	{I2C_DATA_OFF();};
  bool getDataPin()  { return IsPortBit(I2C_DATA_PORT.DR.Byte,I2C_DATA_PIN );};

  void clockIn()   	{I2C_CLK_IN();};
  void clockOut()  	{I2C_CLK_OUT();};
  void clockHigh() 	{I2C_CLK_ON();};
  void clockLow()  	{I2C_CLK_OFF();};


 void TimerStart() 
	{
	   I2C_TIMER.TIER.Bit.TGIEA = 1;
	   I2C_TIMER.TCNT = 0;
	   TpuStart(I2C_TIMER_NO);
	}

 void TimerStop() 
	{ 
	  I2C_TIMER.TIER.Bit.TGIEA = 0;
	  TpuStop(I2C_TIMER_NO);
	}

 void setRegister(regs reg, BYTE value)
 {
   	static BYTE dat[3] = {i2cAddr,reg,value};
   	write(dat,sizeof(dat));
 }

 BYTE getRegister(regs reg)
 {
   return 0;
 }

protected:
  static void OnTimer(void) asm(I2C_ISR) __attribute__ ((interrupt_handler));
  static void OnAlert(void) asm("__Interrupt_IRQ0") __attribute__ ((interrupt_handler));
};

#endif //__LM87_H__
