#ifndef __UART_H__
#define __UART_H__

//SCI
#ifdef __cplusplus
namespace sci_cfg
{
#endif
 //SMR
	#define async	0
	#define	synch	1
	#define even	0
	#define odd		1
	#define MODE(m)		((m&1)<<7)				
	#define CHAR_LEN(x)	(((((~x)&8)>>3))<<6)
	#define	PARITY(p) 	((((p&1)|2)&0x3)<<4)
	#define STOP(n)		(((n>>2)&0x1)<<3)
	#define CLK_PIN(n)	(((n>>2)&0x1)<<3)
 //SCR
	#define SCK_IO	0
	#define SCK_INT_OUT_EXT_IN	1
	#define SCK_INT_IN_EXT_OUT	2

	#define TX(m)		((m&1)<<5)
	#define RX(m)		((m&1)<<4)

	#define SCLK_PIN(m)		(m&0x3)
 //BRR
	#define BAUD(F_MHz,n,B)	((BYTE)(F_MHz/((1L<<(2*n+5))*(ULONG)B)-1))
#ifdef __cplusplus
};
#endif


template<class _T, size_t size> class fifo
{
	size_t count;
	_T* putPtr;
	_T* getPtr;
	_T  queue[size];

 public:
	fifo()
	{
	  count=0;
	  putPtr = getPtr = queue;
	};

	void push(const _T &d)
	{
	  if(count==size)
		return;
	  *putPtr++=d;
	  if(putPtr == &queue[size] )
	      putPtr = queue;
	  count++;
	}

	_T pop()
	{
	  if(count==0)
		return;
	  _T &d = *getPtr++;
	  count--;
	  if(getPtr == &queue[size] )
		getPtr = queue;
	  return d;
	}

	bool empty() {return (count==0);};
	bool full()  {return (count==size);};
	size_t max_size() {return size;}
	size_t size_of() {return count;}
};


template<class _Impl, class _Fifo = fifo<BYTE, 64> > class Uart
{
  _Fifo q;
  BYTE *m_txPtr;
  size_t m_txSize;
  bool m_txFree;

  BYTE m_lastError;
 protected:
	 typedef Uart<_Impl,_Fifo> _Parent;
 public:
	Uart()
	{
	  m_txFree=false;
	  ((_Impl *)this)->enable();
	}

	bool UartTryPut(void *data, size_t size)
	{
	  bool ret=false;
	  ((_Impl *)this)->TxLock();
	  if(m_txFree)
	  {
		  m_txPtr = data;
		  m_txSize = size;
		  m_txFree = false;

		  ((_Impl *)this)->Tx() = *m_txPtr++;
		  m_txSize--;
		  ret = true;
	  }
	  ((_Impl *)this)->TxUnlock();
	  return ret;
	}

	bool UartTryGet(void *data, size_t size)
	{
	  bool ret=false;
	  ((_Impl *)this)->RxLock();
	  if(size>=q.size_of())
	  {
	    for(;size>0;size--)
		  data++ = q.pop();
		ret=true;
	  }
	  ((_Impl *)this)->RxUnlock();
	  return ret;
	}

	void enable() {};
	void disable() {};

	BYTE &Tx() 		{return 0;}
	void TxLock()   {};
	void TxUnlock() {};
	bool TxEmpty()  {return true;}

	void RxLock()   {};
	void RxUnlock() {};
	BYTE &Rx() 		{return 0;}

	void TxRxLock()   {};
	void clearError() {};
	BYTE getErr()	  {return 0;};

	void Complete() {};

protected:
	void OnTransmit()
	{
	   if(--m_txSize==0)
	   {
		 ((_Impl *)this)->TxLock();
		 ((_Impl *)this)->Complete();
		 return;
	   }
	   ((_Impl *)this)->Tx() = *m_txPtr++;
	}

	void OnReceive()
	{
		if(!q.full())
			q.push(((_Impl *)this)->Rx());
	}

	void OnError()
	{
	   TxRxLock();
	   m_lastError = ((_Impl *)this)->getErr();
	   ((_Impl *)this)->Complete();
	}

/*#define UART2_TRANSMIT_ISR    void _Interrupt_TXI2_SCI2( void )
#define UART2_SHIFT_EMTY_ISR  void _Interrupt_TEI2_SCI2( void )
#define UART2_RECEIVE_ISR     void _Interrupt_RXI2_SCI2( void )
#define UART2_ERROR_ISR       void _Interrupt_ERI2_SCI2( void )
*/
};

#endif //__UART_H__
